/* -*-c-*-
 *
 * $Id: mpmont-exp.h,v 1.2 2004/03/21 22:52:06 mdw Exp $
 *
 * Exponentiation operations for Montgomery reduction
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: mpmont-exp.h,v $
 * Revision 1.2  2004/03/21 22:52:06  mdw
 * Merge and close elliptic curve branch.
 *
 * Revision 1.1.4.1  2004/03/20 00:13:31  mdw
 * Projective coordinates for prime curves
 *
 * Revision 1.1  2001/06/16 12:58:12  mdw
 * Parameters for generic exponentiation.
 *
 */

#ifndef CATACOMB_MPMONT_EXP_H
#define CATACOMB_MPMONT_EXP_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Exponentation definitions -----------------------------------------*/

#define EXP_TYPE mp *

#define EXP_COPY(d, x) d = MP_COPY(x)
#define EXP_DROP(x) MP_DROP(x)

#define EXP_MUL(a, x) do {						\
  mp *t = mpmont_mul(mm, spare, a, x);					\
  spare = a;								\
  a = t;								\
} while (0)

#define EXP_SQR(a) do {							\
  mp *t = mp_sqr(spare, a);						\
  spare = a;								\
  a = mpmont_reduce(mm, t, t);						\
} while (0)

#define EXP_FIX(x)

#define EXP_SETMUL(d, x, y) d = mpmont_mul(mm, MP_NEW, x, y)

#define EXP_SETSQR(d, x) do {						\
  d = mp_sqr(MP_NEW, x);						\
  d = mpmont_reduce(mm, d, d);						\
} while (0)

#include "exp.h"

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
